// Data: 10/09/2017 - 1o Semestre de 2024 ps-greve
// Prof. Dr. Jean M. Simo - DAINF/UTFPR Curitiba-Central

// UNIVERSIDADE TECNOLGICA FEDERAL DO PARAN - Campus de Curitiba Central -
// Departamento Acadmico de Informtica (DAINF).
// Disciplina: Tcnicas de Programao - CSE20
// Prova sobre linguagem C++ 2003 / Diagrma de Classes em UML / Orientao a Objetos - Exame de Susbtituio de 1a Parcial

// Curso: Engenharia da Computao, BSI afins.
// Turma: S71.

// Leia toda a prova antes de comear, pois os enunciados podem estar ompletados uns nos outros.
// Utilize os BONS princpios de projeto e programao orientada a objetos.

// Texto da prova.

// A interpretao faz parte do contedo da prova!
// Inclua comentrio (se for o caso) para deixar explcita suas decises.

// Excutado em Dev-C++ 4.9.9.2

#include <iostream>
#include <string>
#include <vector>
#include <stack>
using namespace std;

// classe abstrata
class Personagem
{
    private:
        string nome;
    public:
        Personagem() : nome ("") { }
        virtual ~Personagem() {  nome = ""; }

        const string getNome() const { return nome; }
        void setNome(const string n) { nome = n; }

        virtual void mover() const = 0;
};

class Jogador : public Personagem // Derivado da classe .
{
  	private:
        int vidas;   // Atributo inteiro
  	public:
        Jogador(): Personagem(), vidas (0) { }
        ~Jogador() { vidas = 0; }

        const int getVidas() const { return vidas; }
        void setVidas(const int v) { vidas = v; }

        void mover() const { cout  << getNome() <<  " movendo." << endl; }
};

//classe abstrata
class Protetor : public Personagem
{
  	private:
        float energia;

  	public:
        Protetor(): Personagem(), energia(0)  { }
        virtual ~Protetor() { energia = 0; }

        const float getEnergia() const {return energia; }
        void setEnergia(const float e) { energia = e; }

        virtual void mover() const = 0;
};

// classe abstrata
class Inimigo: public Personagem // Derivado da classe
{
  	private:
        bool ativo;
  	public:
        Inimigo(): Personagem(), ativo (true) {  }
        virtual ~Inimigo() { ativo = false; }

        const bool getAtivo() const { return ativo; }
        void setAtivo(const bool a) { ativo = a; }

        virtual void mover() const = 0;
};

class Curupira : public Protetor
{
    private:
        vector<Personagem*> LPs;
        int idade;
    public:
        Curupira() : Protetor(), LPs(), idade (0) { LPs.clear(); }
        ~Curupira() { LPs.clear(); idade = -1; }

		const int getIdade() const { return idade; }
        void setIdade(const int i) { idade = i; }

        void  proteger_todos() { /* a fazer */ }

        void incluir(Personagem* pc)
        {
         	if (pc) { LPs.push_back(pc); }
            else { cout << "Ponteiro nulo !" << endl; }
        }

        // criado apenas para testar, no precisaria pelas questes...
        void listar() const
        {
            int tam = (int)LPs.size();
			cout << endl;
            for (int i = 0; i < tam; i++) { cout << LPs[i]->getNome() << endl; }
            cout << endl;
        }

        // Redefinio da funo virtual pura acima
        void mover() const { cout  << getNome() <<  " movendo um passo." << endl; }
};

class Apontador // Classe base - para evitar repetio de cdigo nas dela derivadas.
{
  	private:
        Jogador* pJog;   // ponteiro para jogador

  	public:
        Apontador(): pJog(NULL) { }
        ~Apontador() { pJog = NULL; }

        Jogador* getJogador() const { return pJog; }
        void setJogador(Jogador* pj) { pJog = pj; }
};

class Fada : public Protetor, public Apontador
{
    protected:
        string posto;

    public:
        Fada() : Protetor(), Apontador(), posto("") { }
        ~Fada() { posto = ""; }

        const string getPosto() const { return posto; }
        void setPosto(const string p) { posto = p; }

        void protege_jogador() { /* a fazer */ }

        // Redefinio da funo virtual pura acima
        void mover() const { cout << getNome() << " movendo um passo." << endl; }
};


class BoiTata: public Inimigo, public Apontador
{
    protected:
        float poder_fogo;

    public:
        BoiTata() : Inimigo(), Apontador(), poder_fogo (0.0) {  }
        ~BoiTata() { poder_fogo = 0.0; }

        const float getPoderFogo() const { return poder_fogo; }
        void setPoderFogo(const float pf) { poder_fogo = pf; }

        void enviar_fogo() { /* a fazer */ }

        // Redefinio da funo virtual pura acima
        void mover() const { cout  << getNome() <<  " movendo um passo." << endl; }
        void operator++() { poder_fogo++; }
};

class Pilha_Personagens
{
  	private:
        stack<Personagem*> LPs;
	public:

        Pilha_Personagens() { }
        ~Pilha_Personagens() { }

        void empilhar(Personagem* PC)
        {
            if (NULL != PC)
            {
                if (LPs.size() < 10) { LPs.push(PC); }
                else { cout << "Pilha cheia." << endl; }
            }
            else { cout << "Ponteiro de componente invlido - nulo " << endl;}
        }

        void desempilhar()
        {
            if ( !LPs.empty() )
            {
                while (!LPs.empty())
                {
                    Personagem* p = NULL;
                    p = LPs.top();
                    if (p) { p->mover(); }
                    else { cout << "Ponteiro nulo!" << endl;}
                    LPs.pop();
                }
            }
            else { cout << "Pilha Vazia!" << endl << endl; }
        }
};

class Jogo
{
  private:

        Pilha_Personagens P;

        Jogador    J1;
        Fada       F1;
        Curupira   C1;
        BoiTata    B1;

  public:

        Jogo() : P(), J1(), F1(), C1(), B1()
        {
            J1.setNome("J1");
            J1.setVidas(1);

            F1.setNome("F1");
            F1.setEnergia(1.0);
            F1.setPosto("Fada Madrinha");

            C1.setNome("C1");
            C1.setEnergia(10.0);
            C1.setIdade(300);

            B1.setNome("B1");
            B1.setAtivo(true);
            B1.setPoderFogo(1.5);

            P.empilhar(static_cast<Personagem*>(&J1));  // ou apenas P.empilhar(&J1);
            P.empilhar(static_cast<Personagem*>(&F1));
            P.empilhar(static_cast<Personagem*>(&C1));
            //for (int i = 0; i < 10; i++) // soh para testar a pilha...
            P.empilhar(static_cast<Personagem*>(&B1));

            // apenas para testar
            C1.incluir(static_cast<Personagem*>(&J1));
            C1.incluir(static_cast<Personagem*>(&F1));
            C1.incluir(static_cast<Personagem*>(&B1));

            // apenas para testar
            F1.setJogador(&J1);
            B1.setJogador(&J1);

            executar();
           }

           ~Jogo() { }

           void listar() { P.desempilhar();  }

           void executar()
           {
            	listar();
            	// apenas para testar
                C1.listar();
           }
};

int main()
{
    const Jogo Brasileirinho;

    system ( "PAUSE" );

    return 0;
}

